"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.privateStatic = exports.publicInstanceGlobal = exports.privateInstance = exports.publicInstance = void 0;
function set(obj, prop, value, enumerable = true) {
    Object.defineProperty(obj, prop, {
        configurable: true,
        enumerable,
        value
    });
}
function nameFn(name, fn) {
    set(fn, 'name', `@LazyGetter${String(name)}`);
    return fn;
}
function autoNameFn({ name }) {
    return `@LazyGetter(${name})`;
}
/** @internal */
function publicInstance(name, target, select) {
    return nameFn(autoNameFn(target), select
        ? function () {
            const value = target.call(this);
            if (select.call(this, value, this)) {
                set(this, name, value);
            }
            return value;
        }
        : function () {
            const value = target.call(this);
            set(this, name, value);
            return value;
        });
}
exports.publicInstance = publicInstance;
/** @internal */
function privateInstance(name, target, select) {
    const RESULT_COMPUTED = Symbol(`${String(name)} result`);
    return nameFn(autoNameFn(target), select
        ? function () {
            if (RESULT_COMPUTED in this) {
                return this[RESULT_COMPUTED];
            }
            const value = target.call(this);
            if (select.call(this, value, this)) {
                set(this, RESULT_COMPUTED, value, false);
            }
            return value;
        }
        : function () {
            if (RESULT_COMPUTED in this) {
                return this[RESULT_COMPUTED];
            }
            const value = target.call(this);
            set(this, RESULT_COMPUTED, value, false);
            return value;
        });
}
exports.privateInstance = privateInstance;
/** @internal */
function publicInstanceGlobal(name, target, select) {
    return nameFn(autoNameFn(target), select
        ? function () {
            const value = target.call(this);
            if (select.call(this, value, this)) {
                set(Object.getPrototypeOf(this), name, value, false);
            }
            return value;
        }
        : function () {
            const value = target.call(this);
            set(Object.getPrototypeOf(this), name, value, false);
            return value;
        });
}
exports.publicInstanceGlobal = publicInstanceGlobal;
/** @internal */
function privateStatic(target, select) {
    let firstCall = true;
    let value;
    return nameFn(autoNameFn(target), select
        ? function () {
            if (firstCall) {
                value = target.call(this);
                if (select.call(this, value, this)) {
                    firstCall = false;
                }
            }
            return value;
        }
        : function () {
            if (firstCall) {
                value = target.call(this);
                firstCall = false;
            }
            return value;
        });
}
exports.privateStatic = privateStatic;
